package;
import flixel.FlxG;
import flixel.input.gamepad.FlxGamepad;
import flixel.input.gamepad.XboxButtonID;

/**
 * ...
 * @author Ronan Roquais
 */
class ControlPad
{

  private var _keys:Array<Array<String>>; //up, down, left, right, A, B, X, Y, Start
  private var _latestPad:FlxGamepad;
  private var _gId:Int;
  public function new(keys:Array<Array<String>>, controllerId:Int=-1) 
  {
    _keys = new Array();
    _keys[XboxButtonID.DPAD_UP] = keys[0];
    _keys[XboxButtonID.DPAD_DOWN] = keys[1];
    _keys[XboxButtonID.DPAD_LEFT] = keys[2];
    _keys[XboxButtonID.DPAD_RIGHT] = keys[3];
    _keys[XboxButtonID.A] = keys[4];
    _keys[XboxButtonID.B] = keys[5];
    _keys[XboxButtonID.X] = keys[6];
    _keys[XboxButtonID.Y] = keys[7];
    _keys[XboxButtonID.START] = keys[8];
    _gId = controllerId;
    if(_gId>0)
      _latestPad = FlxG.gamepads.getByID(_gId);
  }
  
  public function update(gamepad:FlxGamepad)
  {
    _latestPad = gamepad;
  }
  
  private function getXPadAxis():Float
  {
    #if flash
    return _latestPad.getXAxis(0);
    #else
    return _latestPad.getAxis(0);
    #end
  }
  
  public function getXAxis():Float
  {
    var xAxis:Float = 0.0;
    if (_gId >= 0)
    {
      xAxis = getXPadAxis();
    }
    else
    {
      if (FlxG.keys.anyPressed(_keys[XboxButtonID.DPAD_LEFT]))
        xAxis -= 1.0;
      if (FlxG.keys.anyPressed(_keys[XboxButtonID.DPAD_RIGHT]))
        xAxis += 1.0;
    }
    return xAxis;
  }
  
  private function getYPadAxis():Float
  {
    #if flash
    return -_latestPad.getYAxis(1);
    #else
    return _latestPad.getAxis(1);
    #end
  }
  public function getYAxis():Float
  {
    var yAxis:Float = 0.0;
    if (_gId >= 0)
    {
      yAxis = getYPadAxis();
    }
    else
    {
      if (FlxG.keys.anyPressed(_keys[0]))
        yAxis -= 1.0;
      if (FlxG.keys.anyPressed(_keys[1]))
        yAxis += 1.0;
    }
    return yAxis;
  }
  
  public function isLeftPressed():Bool
  {
    if (_gId >= 0)
    {
      return _latestPad.pressed(XboxButtonID.DPAD_LEFT) && getXPadAxis() > -_latestPad.deadZone;
    }
    else
    {
      return FlxG.keys.anyPressed(_keys[XboxButtonID.DPAD_LEFT]);
    }
  }
  
  public function isRightPressed():Bool
  {
    
    if (_gId >= 0)
    {
      return _latestPad.pressed(XboxButtonID.DPAD_RIGHT) && getXPadAxis() > _latestPad.deadZone;
    }
    else
    {
      return FlxG.keys.anyPressed(_keys[XboxButtonID.DPAD_RIGHT]);
    }
  }
  public function isAPressed():Bool
  {
    if (_gId >= 0)
    {
      return _latestPad.pressed(XboxButtonID.A);
    }
    else
    {
      return FlxG.keys.anyPressed(_keys[XboxButtonID.A]);
    }
  }
  public function isAJustPressed():Bool
  {
    if (_gId >= 0)
    {
      return _latestPad.justPressed(XboxButtonID.A);
    }
    else
    {
      return FlxG.keys.anyJustPressed(_keys[XboxButtonID.A]);
    }
  }
  public function isStartPressed():Bool
  {
    return false;
  }
}