package;

import flixel.FlxG;
import flixel.FlxState;
import flixel.group.FlxGroup;
import flixel.ui.FlxButton;
import flixel.util.FlxDestroyUtil;
import flixel.FlxSprite;
import flixel.input.gamepad.FlxGamepad;
using flixel.util.FlxSpriteUtil;
import flixel.tile.FlxTilemap;
import openfl.Assets;
import flash.geom.ColorTransform;
import flixel.input.gamepad.XboxButtonID;
import flixel.util.FlxColor;


/**
 * A FlxState which can be used for the game's menu.
 */
class MenuState extends FlxState
{
  // How many UI shall we click on before playing
  private var _steps:Int = 2;
  private var _currentStep:Int = 0;

  private var _playerChoiceBtn:Array<Dynamic> = [];
  private var _padChoiceBtn:Array<Dynamic> = [];

  private var _btnPlay:FlxButton;
  private var _btnPlayKey:FlxButton;
  private var _btnPlayer2vs2:FlxButton;
  private var _btnPlayer3vs3:FlxButton;
  private var _btnPlayer4vs4:FlxButton;
  private var _btnStart:FlxButton;
  private var _gamepads:Array<FlxSprite>;
  private var p1Controls:Controls;

  private var _staticBg:FlxSprite;
  /**
   * Function that is called up when to state is created to set it up. 
   */
  private var _activePlayers:Array<Bool>;
  override public function create():Void
  {
    _staticBg = new FlxSprite();
    _staticBg.loadGraphic("assets/images/game-start.png");
    add(_staticBg);

    _activePlayers = [false, false, false, false];

    _btnPlay = new FlxButton(20, 630, "", clickPlay);
    _btnPlay.loadGraphic("assets/images/playGamepad.png", true, 126, 70);
    add(_btnPlay);

    _btnPlayKey = new FlxButton(150, 630, "", clickPlayKey);
    _btnPlayKey.loadGraphic("assets/images/playKey.png", true, 126, 70);
    add(_btnPlayKey);

    _btnStart = new FlxButton(500, 600, "", clickStart);
    _btnStart.loadGraphic("assets/images/playBtn.png", true, 166, 90);
    _btnStart.alpha = 0.4;
    _btnStart.active = false;
    add(_btnStart);

    _gamepads = new Array<FlxSprite>();
    var gamepad = new FlxSprite(10, FlxG.worldBounds.bottom - 150);
    gamepad.makeGraphic(150 , 144, FlxColor.CRIMSON);
    gamepad.visible = false;
    add(gamepad);
    _gamepads.push(gamepad);
    
    gamepad = new FlxSprite(350, FlxG.worldBounds.bottom - 150);
    gamepad.makeGraphic(150, 144, FlxColor.BLUE);
    gamepad.visible = false;
    add(gamepad);
    _gamepads.push(gamepad);
    
    gamepad = new FlxSprite(700, FlxG.worldBounds.bottom - 150);
    gamepad.makeGraphic(150, 144, FlxColor.YELLOW);
    gamepad.visible = false;
    add(gamepad);
    _gamepads.push(gamepad);
    
    gamepad = new FlxSprite(900, FlxG.worldBounds.bottom - 150);
    gamepad.makeGraphic(150, 144, FlxColor.FOREST_GREEN);
    gamepad.visible = false;
    add(gamepad);
    _gamepads.push(gamepad);
    
    var _padChoice:Array<Dynamic> = [_btnPlay, _btnPlayKey];

    for (i in 0..._padChoice.length)
    {
      _padChoiceBtn.push(i);
      _padChoiceBtn[i] = {
        activate: false,
        btn: _padChoice[i]
      };
    }

    createAmountOfPlayersBtn();
    super.create();
  }

  private function createAmountOfPlayersBtn():Void
  {
    _btnPlayer2vs2 = new FlxButton(870, 630, "", clickPlayer2vs2);
    _btnPlayer3vs3 = new FlxButton(1000, 630, "", clickPlayer3vs3);
    _btnPlayer4vs4 = new FlxButton(1130, 630, "", clickPlayer4vs4);

    _btnPlayer2vs2.loadGraphic("assets/images/play2players.png", true, 126, 70);
    _btnPlayer3vs3.loadGraphic("assets/images/play3players.png", true, 126, 70);
    _btnPlayer4vs4.loadGraphic("assets/images/play4players.png", true, 126, 70);

    add(_btnPlayer2vs2);
    add(_btnPlayer3vs3);
    add(_btnPlayer4vs4);

    var _playerChoice:Array<Dynamic> = [_btnPlayer2vs2, _btnPlayer3vs3, _btnPlayer4vs4];

    for (i in 0..._playerChoice.length)
    {
      _playerChoiceBtn.push(i);
      _playerChoiceBtn[i] = {
        activate: false,
        btn: _playerChoice[i]
      };
    }
  }

  private function clickPlayer2vs2():Void
  {
    _currentStep++;
    Reg.player_amount = 2;
    checkBtn(true);
    activateButton(0, true);
  }

  private function clickPlayer3vs3():Void
  {
    _currentStep++;
    Reg.player_amount = 3;
    checkBtn(true);
    activateButton(1, true);
  }

  private function clickPlayer4vs4():Void
  {
    _currentStep++;
    Reg.player_amount = 4;
    checkBtn(true);
    activateButton(2, true);
  }

  private function clickStart():Void 
  {
    FlxG.switchState(new PlayState());
    _btnStart.active = false;
  }
  
  private function clickPlay():Void
  {
    _currentStep++;
    Reg.usePads = true;
    checkBtn();
    activateButton(0);
  }
  private function clickPlayKey():Void
  {
    _currentStep++;
    Reg.usePads = false;
    checkBtn();
    activateButton(1);
  }

  private function activateButton(id:Int, isPlayer:Bool = false):Void 
  {
    if (isPlayer)
    {
      _playerChoiceBtn[id].active = true;
    } 
    else 
    {
      _padChoiceBtn[id].active = true;
    }
  }

  private function checkBtn(isPlayer:Bool = false):Void 
  {
    if (isPlayer)
    {
      for (i in 0..._playerChoiceBtn.length)
      {
        if (_playerChoiceBtn[i].active) 
        {
          _currentStep--;
          _playerChoiceBtn[i].active = false;
          _playerChoiceBtn[i].btn.setColorTransform();
        }
      }
    } else 
    {
      for (i in 0..._padChoiceBtn.length)
      {
        if (_padChoiceBtn[i].active) 
        {
          _currentStep--;
          _padChoiceBtn[i].active = false;
          _padChoiceBtn[i].btn.setColorTransform();
        }
      }
    }
    if (_currentStep >= _steps)
    {
      _btnStart.alpha = 1;
      _btnStart.active = true;
    }
  }
  
  /**
   * Function that is called when this state is destroyed - you might want to 
   * consider setting all objects this state uses to null to help garbage collection.
   */
  override public function destroy():Void
  {
    _btnPlay = FlxDestroyUtil.destroy(_btnPlay);
    _btnPlayKey = FlxDestroyUtil.destroy(_btnPlayKey);
    for(i in 0..._gamepads.length)
    {
      _gamepads[i] = FlxDestroyUtil.destroy(_gamepads[i]);
    }
    super.destroy();
  }

  /**
   * Function that is called once every frame.
   */
  override public function update():Void
  {
    var gIds:Array<Int> = new Array<Int>();
    gIds = FlxG.gamepads.getActiveGamepadIDs();
    
    var gamepads:Array<FlxGamepad> = FlxG.gamepads.getActiveGamepads();
    var i:Int = 0;
    while (i < gIds.length)
    {
      if (gamepads[i].justPressed(XboxButtonID.START))
      {
        _activePlayers[i] = true;
      }
      i++;
    }
    var i:Int;
    for (i in 0...Reg.playerKeys.length)
    {
      if (!_activePlayers[i] && FlxG.keys.anyJustPressed(Reg.playerKeys[i][8])) // start in reg
      {
        _activePlayers[i] = true;
      }
      if (_activePlayers[i])
      {
        _gamepads[i].visible = true;
      }
    }
    super.update();
  } 
}