import flixel.effects.particles.FlxEmitter;
import flixel.FlxG;
import flixel.FlxObject;
import flixel.FlxState;
import flixel.FlxSprite;
import flixel.group.FlxGroup;
import flixel.text.FlxText;
import flixel.tile.FlxTilemap;
import flixel.util.FlxColor;
import flixel.util.FlxPoint;
import flixel.tweens.FlxTween;
import flixel.tweens.FlxEase;
import openfl.Assets;

class PlayState extends FlxState
{
  public var map:FlxTilemap;
  public var players:FlxGroup;
  public var levelHolder:LevelHolder;
  private var _restart:Bool;
  private var currentGoalId:Int = -1;
  private var playerScore:Array<Int> = new Array<Int>();
  private var _hud:HUD;
  private var playerRank:Array<Player>;

  override public function create():Void
  {
    FlxG.gamepads.globalDeadZone = 0.3;
    FlxG.camera.bgColor = 0xff4444ff;
    
    map = new FlxTilemap();
    map.allowCollisions = FlxObject.ANY;
   
    _restart = false;
    levelHolder = new LevelHolder(this);

    add(map.loadMap(Assets.getText("assets/levels/0/level.csv"), "assets/art/ice-sprites.png", 20, 20));

    createPlayers();

    _hud = new HUD();
    add(_hud);

    super.create();
  }
  
  override public function destroy():Void
  {
    map.destroy();
    playerRank = null;
    players.destroy();
    
    _hud.destroy();
  }
  
  override public function update():Void 
  {
    FlxG.collide(players, map);
    FlxG.collide(players, players);
    for (post in levelHolder.goalPosts.posts)
    {
      if (post.active) 
        FlxG.overlap(players, post, collectGoalPost);
    }
    FlxG.overlap(players, levelHolder.levelEntities.levelEntities, touchLevelEntity);
    checkPlayers();
    super.update();
  }

  private function createPlayers():Void
  {
    players = new FlxGroup();
    playerScore = [];
    playerRank = new Array<Player>();
    var player:Player;
    for (i in 0...Reg.player_amount) 
    {
      player = new Player(
        levelHolder.playerPos.rows[i].x, 
        levelHolder.playerPos.rows[i].y, 
        this, i);
      players.add(player);
      playerRank.push(player);
      add(players);
      playerScore.push(i);
      playerScore[i] = 0;
    }
  }

  private function checkPlayers():Void
  {
    var player:Player;
    for (i in 0...playerRank.length) 
    {
      player = cast (playerRank[i], Player);
      if (!player.alive)
      {
        player.x = levelHolder.playerPos.rows[i].x;
        player.y = levelHolder.playerPos.rows[i].y;
        player.revive();
      }
    }
  }

  private function setPlayers():Void
  {
    for (i in 0...playerRank.length) 
    {
      playerRank[i].x = levelHolder.playerPos.rows[i].x;
      playerRank[i].y = levelHolder.playerPos.rows[i].y;
      playerRank[i].revive();
    }
  }

  private function reorganiseRank(G:GoalPost):Void 
  {
    var player:Player;
    var dist:Float;
    var a:Array<Int> = new Array<Int>();
    var map:Map<Int, Int> = new Map<Int, Int>();
    for (i in 0...players.members.length) 
    {
      player = cast (players.members[i], Player);
      dist = Math.sqrt(
        (player.x - G.x) * (player.x - G.x) +
        (player.y - G.y) * (player.y - G.y)
      );
      a.push(cast (dist, Int));
      map.set(cast (dist, Int), i);
    }
    a.sort(function(a,b) return a-b);

    for (i in 0...a.length)
    {
      if (map.get(a[i]) != null) 
      {
        playerRank.push(cast (players.members[map.get(a[i])], Player));
      }
    }
  }
  
  private function collectGoalPost(P:Player, G:GoalPost):Void
  {
    if (currentGoalId != G.id) {
      playerRank = new Array<Player>();
      playerScore[P.id]++;
      levelHolder.goalPosts.deactivate(G);
      currentGoalId = G.id;
      if (playerScore[P.id] < 3)
      {
        _hud.updateHUD(P.id, playerScore[P.id]);
        reorganiseRank(G);
        for (player in players)
        {
          player.kill();
        }
        levelHolder.currentDirection = (levelHolder.currentDirection > 0) ? 0 : 1;
        var player = cast (P, Player);
        player.score++;
        setPlayers();
        var goal:GoalPost;
        for (post in levelHolder.goalPosts.posts)
        {
          goal = cast (post, GoalPost);
          if (goal.id != currentGoalId) {
            levelHolder.goalPosts.activate(goal);
            break;
          }
        }
        var step:Int = Math.floor(Math.random() * 4);
        levelHolder.setStepPositions(step);
      }
      else 
      {
        Reg.player_id = P.id;
        FlxG.switchState(new WinningState());
      }
    }
  }

  private function touchLevelEntity(P:Player, G:LevelEntity):Void
  {
    G.onTouchAction(P);
  }
}