package;

import flixel.input.gamepad.FlxGamepad;
import openfl.Assets;
import flixel.effects.particles.FlxEmitter;
import flixel.FlxG;
import flixel.group.FlxGroup;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.FlxG;

class Player extends FlxSprite 
{
  @:isVar public var score(get, set):Int;
  @:isVar public var id(get, set):Int;

  public static inline var RUN_SPEED:Int = 340;
  public static inline var GRAVITY:Int = 600;
  public static inline var JUMP_SPEED:Int = 300;
  public static inline var JUMPS_ALLOWED:Int = 2;
  
  private var _gibs:FlxEmitter;
  private var _cooldown:Float;
  private var _parent:PlayState;
  
  private var _jumpTime:Float = -1;
  private var _timesJumped:Int = 0;
  private var _jumpKeys:Array<String>;
  
  private var _xgridleft:Int = 0;
  private var _xgridright:Int = 0;
  private var _ygrid:Int = 0;
  
  public var climbing:Bool = false;
  private var _onLadder:Bool = false;
  
  private var _bonusTime:Float = 1.5;
  private var _currentBonusTime:Float = -1;
  
  private var _controls:Controls;
  
  public function new(X:Float, Y:Float, Parent:PlayState, Id:Int) 
  {
    super(X, Y);
    this.id = Id;

    var idPad = -1;
    if (Reg.usePads)
    {
      idPad = Id;
    }
    _controls = new Controls(Reg.playerKeys[Id], idPad);
    var playerPath:String = "";
    if (Id == 0)
      playerPath = "assets/art/playerRed.png";
    else if (Id == 1)
      playerPath = "assets/art/playerPurple.png";
    else if (Id == 2)
      playerPath = "assets/art/playerYellow.png";
    else if (Id == 3)
      playerPath = "assets/art/playerGreen.png";
    loadGraphic(playerPath, true, 30, 27);
    
    animation.add("walking", [0, 1, 2, 3], 12, true);
    animation.add("idle", [3]);
    animation.add("jump", [4]);
    
    drag.set(RUN_SPEED * 8, RUN_SPEED * 8);
    maxVelocity.set(RUN_SPEED, JUMP_SPEED);
    acceleration.y = GRAVITY;
    setSize(25, 27);
    offset.set(2, 0);
    _parent = Parent;
  }
  
  public override function update():Void
  {
    // Reset to 0 when no button is pushed
    acceleration.x = 0; 
    
    if (climbing) 
    {
      // Stop falling if you're climbing a ladder
      acceleration.y = 0;  
    }
    else 
    {
      acceleration.y = GRAVITY;
    }
    var xAxis:Float=0.0;
    var yAxis:Float=0.0;
    
    var gamepad:ControlPad = _controls.getGamepad();
    var ids:Array<Int> = FlxG.gamepads.getActiveGamepadIDs();
    xAxis = gamepad.getXAxis();
    yAxis = gamepad.getYAxis();

    if (xAxis<0)
    {
      flipX = true;
    }
    else if (xAxis>0)
    {
      flipX = false;  
    }
    
    if (_currentBonusTime > 0)
    {
      if(_currentBonusTime < _bonusTime)
      {
        maxVelocity.set(maxVelocity.x * 1.2, maxVelocity.y);
        _currentBonusTime+= FlxG.elapsed;
      }
      else
      {
        _currentBonusTime = -1;
      }
    }
    else
    {
      maxVelocity.set(RUN_SPEED, maxVelocity.y);
    }
    acceleration.x = xAxis*drag.x;

    jump(gamepad);

    // Animations
    if (velocity.x > 0 || velocity.x < 0) 
    { 
      animation.play("walking"); 
    }
    else if (velocity.x == 0) 
    { 
      animation.play("idle"); 
    }
    if (velocity.y < 0) 
    { 
      animation.play("jump"); 
    }
    
    if (isTouching(FlxObject.FLOOR) && !gamepad.isAPressed())
    {
      _jumpTime = -1;
      // Reset the double jump flag
      _timesJumped = 0;
    }
    
    if (x < FlxG.worldBounds.left)
      x = FlxG.worldBounds.left;
    else if (x > FlxG.worldBounds.right-width*1.5)
      x = FlxG.worldBounds.right-width*1.5;
    if(y > FlxG.worldBounds.bottom)
      kill();
    super.update();
  }
  
  private function jump(gamepad:ControlPad):Void
  {
    if (gamepad.isAJustPressed())
    {
      if ((velocity.y == 0) || (_timesJumped < JUMPS_ALLOWED)) // Only allow two jumps
      {
        FlxG.sound.play("assets/sounds/jump" + Reg.SoundExtension, 1, false);
        if (!isTouching(FlxObject.FLOOR))
          _timesJumped++;
        _timesJumped++;
        _jumpTime = 0;
        _onLadder = false;
      }
    }
    
    // You can also use space or any other key you want
    if ((gamepad.isAPressed()) && (_jumpTime >= 0)) 
    {
      climbing = false;
      _jumpTime += FlxG.elapsed;
      
      // You can't jump for more than 0.25 seconds
      if (_jumpTime > 0.25)
      {
        _jumpTime = -1;
      }
      else if (_jumpTime > 0)
      {
        velocity.y = - 0.6 * maxVelocity.y;
      }
    }
  }
  
  public function bounce():Void
  {
    velocity.y = -maxVelocity.y;
  }
  
  public function speedUp():Void
  {
    _currentBonusTime = FlxG.elapsed;
  }
  
  override public function kill():Void
  {
    if (!alive) 
    {
      return;
    }

    super.kill();
    
    // FlxG.sound.play("assets/sounds/death" + Reg.SoundExtension, 1, false);
  }

  function get_id() 
  {
    return id;
  }

  function set_id(id) 
  {
    return this.id = id;
  }

  function get_score() 
  {
    return score;
  }

  function set_score(score) 
  {
    return this.score = score;
  }
}